@@ -1,4 +1,26 @@
-/* #includes */ /*{{{C}}}*//*{{{*/
+/* Notes */ /*{{{C}}}*//*{{{*/
+/*
+
+This program is GNU software, copyright 1997, 1998, 1999, 2000, 2001
+Michael Haardt <michael@moria.de>.
+
+This program is free software; you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation; either version 2 of the License, or (at your
+option) any later version.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program.  If not, write to the Free Software Foundation, Inc.,
+59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+*/
+/*}}}*/
+/* #includes */ /*{{{*/
 #undef  _POSIX_SOURCE
 #define _POSIX_SOURCE   1
 #undef  _POSIX_C_SOURCE
@@ -371,6 +393,7 @@ int main(int argc, char *argv[]) /*{{{*/
   {
     FILE *fp;
     if ((fp=fopen(argv[optind],"r"))==(FILE*)0) fprintf(stderr,OPENFAILED,argv[optind],strerror(errno));
+    else
     {
       sentence("diction",catd,fp,argv[optind],diction,phraseLanguage);
       fclose(fp);
