@@ -1,4 +1,26 @@
-/* #includes */ /*{{{C}}}*//*{{{*/
+/* Notes */ /*{{{C}}}*//*{{{*/
+/*
+
+This program is GNU software, copyright 1997, 1998, 1999, 2000, 2001
+Michael Haardt <michael@moria.de>.
+
+This program is free software; you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation; either version 2 of the License, or (at your
+option) any later version.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program.  If not, write to the Free Software Foundation, Inc.,
+59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+*/
+/*}}}*/
+/* #includes */ /*{{{*/
 #ifndef NO_POSIX_SOURCE
 #undef  _POSIX_SOURCE
 #define _POSIX_SOURCE   1
@@ -64,7 +86,16 @@ static const char *abbreviations_en[]= /*{{{*/
   "Figs",
   "figs",
   "ft",
-  "in",
+  "0 in",
+  "1 in",
+  "2 in",
+  "3 in",
+  "4 in",
+  "5 in",
+  "6 in",
+  "7 in",
+  "8 in",
+  "9 in",
   "Inc",
   "Jr",
   "jr",
@@ -98,6 +129,7 @@ static int endingInAbbrev(const char *s, size_t length) /*{{{*/
 {
   const char **abbrev=abbreviations;
   size_t aLength;
+
   if (!isalpha(s[length-1])) return 0;
   while (*abbrev!=(const char*)0)
   {
