@@ -1,4 +1,26 @@
-/* #includes */ /*{{{C}}}*//*{{{*/
+/* Notes */ /*{{{C}}}*//*{{{*/
+/*
+
+This program is GNU software, copyright 1997, 1998, 1999, 2000, 2001
+Michael Haardt <michael@moria.de>.
+
+This program is free software; you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation; either version 2 of the License, or (at your
+option) any later version.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program.  If not, write to the Free Software Foundation, Inc.,
+59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+*/
+/*}}}*/
+/* #includes */ /*{{{*/
 #undef  _POSIX_SOURCE
 #define _POSIX_SOURCE   1
 #undef  _POSIX_C_SOURCE
@@ -33,31 +55,35 @@ static void catclose(nl_catd catd) { }
 #define USAGE3     catgets(catd,1, 3,"       style --version\n")
 #define MORE       catgets(catd,1, 4,"Try style -h|--help for more information.\n")
 #define HELP1      catgets(catd,1, 5,"Analyse surface characteristics of a document.\n")
-#define HELP2      catgets(catd,1, 6,"-L, --language    set the document language.\n")
-#define HELP3      catgets(catd,1, 7,"-l, --print-long  print all sentences longer than <length> words\n")
-#define HELP4      catgets(catd,1, 8,"-r, --print-ari   print all sentences with an ARI greater than than <ari>\n")
-#define HELP5      catgets(catd,1, 9,"-h, --help        print this message\n")
-#define HELP6      catgets(catd,1,10,"    --version     print the version\n")
-#define OPENFAILED catgets(catd,1,11,"style: can not open %s: %s\n")
-#define NOSENT     catgets(catd,1,12,"No sentences found.\n")
-#define NOMEM      catgets(catd,1,13,"style: out of memory\n")
-#define GRADES     catgets(catd,1,14,"readability grades:\n")
-#define KINCAID    catgets(catd,1,15,"Kincaid")
-#define ARI        catgets(catd,1,16,"ARI")
-#define CL         catgets(catd,1,17,"Coleman-Liau")
-#define FLESCH     catgets(catd,1,18,"Flesch Index")
-#define FOG        catgets(catd,1,19,"Fog Index")
-#define WSFT       catgets(catd,1,20,"1. WSTF Index")
-#define WSI        catgets(catd,1,21,"Wheeler-Smith Index")
-#define BELOW5     catgets(catd,1,22,"below school year 5\n")
-#define AHEAD10    catgets(catd,1,23,"higher than school year 10\n")
-#define YEAR       catgets(catd,1,24,"school year %d\n")
-#define LIX        catgets(catd,1,25,"Lix")
-#define AHEAD11    catgets(catd,1,26,"higher than school year 11\n")
-#define SMOG       catgets(catd,1,27,"SMOG-Grading")
-#define SENTINFO   catgets(catd,1,28,"sentence info:\n")
-#define CHARS      catgets(catd,1,29,"        %d characters\n")
-#define SHORTSENT  catgets(catd,1,30,"        %d%% (%d) short sentences (at most %d words)\n")
+#define HELP2      catgets(catd,1, 6,"-L, --language          set the document language.\n")
+#define HELP3      catgets(catd,1, 7,"-l, --print-long        print all sentences longer than <length> words\n")
+#define HELP4      catgets(catd,1, 8,"-r, --print-ari         print all sentences with an ARI greater than than <ari>\n")
+#define HELP5      catgets(catd,1, 9,"-p, --print-passive     print all sentences phrased in the passive voice\n")
+#define HELP6      catgets(catd,1,10,"-N, --print-nom         print all sentences containing nominalizations\n")
+#define HELP7      catgets(catd,1,11,"-n, --print-nom-passive print all sentences phrased in the passive voice or\n                        containing nominalizations\n")
+#define HELP8      catgets(catd,1,12,"-h, --help              print this message\n")
+#define HELP9      catgets(catd,1,13,"    --version           print the version\n")
+#define BUGS       catgets(catd,1,14,"Report bugs to <michael@moria.de>.\n")
+#define OPENFAILED catgets(catd,1,15,"style: can not open %s: %s\n")
+#define NOSENT     catgets(catd,1,16,"No sentences found.\n")
+#define NOMEM      catgets(catd,1,17,"style: out of memory\n")
+#define GRADES     catgets(catd,1,18,"readability grades:\n")
+#define KINCAID    catgets(catd,1,19,"Kincaid")
+#define ARI        catgets(catd,1,20,"ARI")
+#define CL         catgets(catd,1,21,"Coleman-Liau")
+#define FLESCH     catgets(catd,1,22,"Flesch Index")
+#define FOG        catgets(catd,1,23,"Fog Index")
+#define WSFT       catgets(catd,1,24,"1. WSTF Index")
+#define WSI        catgets(catd,1,25,"Wheeler-Smith Index")
+#define BELOW5     catgets(catd,1,26,"below school year 5\n")
+#define AHEAD10    catgets(catd,1,27,"higher than school year 10\n")
+#define YEAR       catgets(catd,1,28,"school year %d\n")
+#define LIX        catgets(catd,1,29,"Lix")
+#define AHEAD11    catgets(catd,1,30,"higher than school year 11\n")
+#define SMOG       catgets(catd,1,31,"SMOG-Grading")
+#define SENTINFO   catgets(catd,1,32,"sentence info:\n")
+#define CHARS      catgets(catd,1,33,"        %d characters\n")
+#define SHORTSENT  catgets(catd,1,34,"        %d%% (%d) short sentences (at most %d words)\n")
 /*}}}*/
 
 /* variables */ /*{{{*/
@@ -66,24 +92,7 @@ static const char *lc_ctype;
 enum lc_ctype_int { ASCII, ISO_8859_1 };
 static enum lc_ctype_int lc_ctype_int;
 static const char *docLanguage;
-
-static int characters;
-static int syllables;
-static int words;
-static int shortwords;
-static int longwords;
-static int bigwords;
-static int sentences;
-static int questions;
-static int articles;
-static int pronouns;
-static int interrogativePronouns;
-static int shortestLine,shortestLength;
-static int longestLine,longestLength;
-static int paragraphs;
-static int printLongSentences=0;
-static float printARI=0.0;
-static struct Hit lengths;
+static const char *phraseEnd = (const char*)0;
 /*}}}*/
 
 /* hit counting functions */ /*{{{*/
@@ -185,7 +194,7 @@ static double fog(int words, int bigwords, int sentences) /*{{{*/
 /*}}}*/
 
 /**
- * Calculate 1. neue Wiener Sachtextformel (WSTF).
+ * Calculate 1. neue Wiener Sachtextformel (WSTF).  MIGHT BE WRONG!
  * @param words the number of words in the text
  * @param shortwords the number of words that contain one syllable
  * @param longwords the number of words that are longer than 6 characters
@@ -199,7 +208,7 @@ static double wstf(int words, int shortwords, int longwords, int bigwords, int s
 /*}}}*/
 
 /**
- * Calculate Wheeler-Smith formula.
+ * Calculate Wheeler-Smith formula.  MIGHT BE WRONG!
  * @param words the number of words in the text
  * @param bigwords the number of words that contain more than 3 syllables
  * @param sentences number of sentences
@@ -254,23 +263,25 @@ static double lix(int *grade, int words, int longwords, int sentences) /*{{{*/
  */
 static double smog(int bigwords, int sentences) /*{{{*/
 {
-  if (strncmp(docLanguage,"de",2)==0) return sqrt(((double)bigwords)/((double)sentences)*30)-2.0;
-  else return sqrt((double)bigwords/((double)sentences)*30)+3.0;
+  if (strncmp(docLanguage,"de",2)==0) return sqrt((((double)bigwords)/((double)sentences))*30)-2.0;
+  else return sqrt((((double)bigwords)/((double)sentences))*30.0)+3.0;
 }
 /*}}}*/
 /*}}}*/
 /* word class checks */ /*{{{*/
-static int cistrncmp(const char *r, const char *s, size_t n)
+static int wordcmp(const char *r, const char *s) /*{{{*/
 {
   int res;
 
-  while (n)
+  while (*r)
   {
-    if ((res=tolower(*r)-tolower(*s))!=0) return res;
-    ++r; ++s; --n;
+    if ((res=*r-tolower(*s))!=0) return res;
+    ++r; ++s;
   }
-  return 0;
+  return isalpha(*s);
 }
+/*}}}*/
+
 /**
  * Test if the word is an article.  This function uses docLanguage to
  * determine the used language.
@@ -292,53 +303,55 @@ static int article(const char *word, size_t l) /*{{{*/
 
   if (strncmp(docLanguage,"de",2)==0) list=de;
   else list=en;
-  while (*list) if (cistrncmp(*list,word,strlen(*list))==0) return 1; else ++list;
+  while (*list) if (wordcmp(*list,word)==0) return 1; else ++list;
   return 0;
 }
 /*}}}*/
+
 /**
  * Test if the word is a pronoun.  This function uses docLanguage to
  * determine the used language.
  */
 static int pronoun(const char *word, size_t l) /*{{{*/
 {
-  static const char *de[]= /* German pronouns */ /*{{{*/
+  static const char *de[]= /* Pronomen */ /*{{{*/
   {
     "ich", "du", "er", "sie", "es", "wir", "ihr", "mein", "meine", "dein",
     "deine", "sein", "seine", "unser", "unsere", "euer", "eure", "mir",
     "mich", "dir", "dich", "ihre", (const char*)0
   };
   /*}}}*/
-  static const char *en[]= /* English pronouns */ /*{{{*/
+  static const char *en[]= /* pronouns */ /*{{{*/
   {
     "i", "me", "we", "us", "you", "he", "him", "she", "her", "it", "they",
     "them", "thou", "thee", "ye", "myself", "yourself", "himself",
     "herself", "itself", "ourselves", "yourselves", "themselves",
-    "oneself", "my", "mine", "his", "hers", "yours", "ours", "theirs",
-    (const char*)0
+    "oneself", "my", "mine", "his", "hers", "yours", "ours", "theirs", "its",
+    "our", "that", "their", "these", "this", "those", "your", (const char*)0
   };
   /*}}}*/
   const char **list;
 
   if (strncmp(docLanguage,"de",2)==0) list=de;
   else list=en;
-  while (*list) if (cistrncmp(*list,word,strlen(*list))==0) return 1; else ++list;
+  while (*list) if (wordcmp(*list,word)==0) return 1; else ++list;
   return 0;
 }
 /*}}}*/
+
 /**
  * Test if the word is an interrogative pronoun.  This function uses
  * docLanguage to determine the used language.
  */
 static int interrogativePronoun(const char *word, size_t l) /*{{{*/
 {
-  static const char *de[]= /* German interrogative pronouns */ /*{{{*/
+  static const char *de[]= /* Interrogativpronomen */ /*{{{*/
   {
     "wer", "was", "wem", "wen", "wessen", "wo", "wie", "warum", "weshalb",
     "wann", "wieso", "weswegen", (const char*)0
   };
   /*}}}*/
-  static const char *en[]= /* English interrogative pronouns */ /*{{{*/
+  static const char *en[]= /* interrogative pronouns */ /*{{{*/
   {
     "why", "who", "what", "whom", "when", "where", "how", (const char*)0
   };
@@ -347,7 +360,201 @@ static int interrogativePronoun(const char *word, size_t l) /*{{{*/
 
   if (strncmp(docLanguage,"de",2)==0) list=de;
   else list=en;
-  while (*list) if (cistrncmp(*list,word,strlen(*list))==0) return 1; else ++list;
+  while (*list) if (wordcmp(*list,word)==0) return 1; else ++list;
+  return 0;
+}
+/*}}}*/
+
+static int conjunction(const char *word, size_t l) /*{{{*/
+{
+  static const char *de[]= /* Konjunktionen */ /*{{{*/
+  {
+    "und", "oder", "aber", "sondern", "doch", "nur", "blo", "denn",
+    "weder", "noch", "sowie", (const char*)0
+  };
+  /*}}}*/
+  static const char *en[]= /* conjunctions */ /*{{{*/
+  {
+    "and", "but", "or", "yet", "nor", (const char*)0
+  };
+  /*}}}*/
+  const char **list;
+
+  if (strncmp(docLanguage,"de",2)==0) list=de;
+  else list=en;
+  while (*list) if (wordcmp(*list,word)==0) return 1; else ++list;
+  return 0;
+}
+/*}}}*/
+
+static int nominalization(const char *word, size_t l) /*{{{*/
+{
+  static const char *de[]= /* Nominalisierungsendungen */ /*{{{*/
+  {
+    "ung", "heit", "keit", "nis", "tum", (const char*)0
+  };
+  /*}}}*/
+  static const char *en[]= /* nominalization suffixes */ /*{{{*/
+  {
+     /* a bit limited, but it is exactly what the original style(1) did */
+     "tion", "ment", "ence", "ance", (const char*)0
+  };
+  /*}}}*/
+  const char **list;
+
+  /* exclude words too short to have such long suffixes */
+  if (l < 7) return 0;
+
+  if (strncmp(docLanguage,"de",2)==0) list=de;
+  else list=en;
+  while (*list) if (wordcmp(*list,word+l-strlen(*list))==0) return 1; else ++list;
+  return 0;
+}
+/*}}}*/
+
+static int subConjunction(const char *word, size_t l) /*{{{*/
+{
+  static const char *de[]= /* unterordnende Konjunktionen */ /*{{{*/
+  {
+    "weil", "da", "dadurch", "wenn", "falls", "sofern", "obwohl",
+    "obgleich", "als", "nachdem", "whrend", "wie", "ob", "je",
+    "desto", "damit", "dass", "indem", "um zu", (const char*)0
+  };
+  /*}}}*/
+  static const char *en[]= /* subordinating conjunctions */ /*{{{*/
+  {
+    "after","because", "lest", "till", "'til", "although", "before", 
+    "now that", "unless", "as", "even if", "provided that", "provided", 
+    "until", "as if", "even though", "since", "as long as", "so that",
+    "whenever", "as much as", "if", "than", "as soon as", "inasmuch",
+    "in order that", "though", "while", (const char*)0
+  };
+  /*}}}*/
+  const char **list;
+
+  if (strncmp(docLanguage,"de",2)==0) list=de;
+  else list=en;
+  while (*list)
+  {
+    if (wordcmp(*list,word)==0) 
+    {
+      phraseEnd = word+strlen(*list);
+      return 1; 
+    }
+    else ++list;
+  }
+  return 0;
+}
+/*}}}*/
+
+static int preposition(const char *word, size_t l) /*{{{*/
+{
+  static const char *de[]= /* Prpositionen */ /*{{{*/
+  {
+    "aus", "auer", "bei", "mit", "nach", "seit", "von", "zu",
+    "bis", "durch", "fr", "gegen", "ohne", "um", "an", "auf",
+    "hinter", "in", "neben", "ber", "unter", "vor", "zwischen",
+    "anstatt", "statt", "trotz", "whrend", "wegen", (const char*)0
+  };
+  /*}}}*/
+  static const char *en[]= /* prepositions */ /*{{{*/
+  {
+    "aboard", "about", "above", "according to", "across from",
+    "after", "against", "alongside", "alongside of", "along with",
+    "amid", "among", "apart from", "around", "aside from", "at", "away from",
+    "back of", "because of", "before", "behind", "below", "beneath", "beside",
+    "besides", "between", "beyond", "but", "by means of",
+    "concerning", "considering", "despite", "down", "down from", "during",
+    "except", "except for", "excepting for", "from among",
+    "from between", "from under", "in addition to", "in behalf of",
+    "in front of", "in place of", "in regard to", "inside of", "inside",
+    "in spite of", "instead of", "into", "like", "near to", "off",
+    "on account of", "on behalf of", "onto", "on top of", "on", "opposite",
+    "out of", "out", "outside", "outside of", "over to", "over", "owing to",
+    "past", "prior to", "regarding", "round about", "round",
+    "since", "subsequent to", "together", "with", "throughout", "through",
+    "till", "toward", "under", "underneath", "until", "unto", "up",
+    "up to", "upon", "with", "within", "without", "across", "along",
+    "by", "of", "in", "to", "near", "of", "from",  (const char*)0
+  };
+  /*}}}*/
+  const char **list;
+
+  if (strncmp(docLanguage,"de",2)==0) list=de;
+  else list=en;
+  while (*list)
+  {
+    if (wordcmp(*list,word)==0)
+    {
+      phraseEnd = word+strlen(*list);
+      return 1; 
+    }
+    else ++list;
+  }
+  return 0;
+}
+/*}}}*/
+
+static int auxVerb(const char *word, size_t l) /*{{{*/
+{
+  static const char *de[]= /* Hilfsverben */ /*{{{*/
+  {
+    "haben", "habe", "hast", "hat", "habt", "gehabt", "htte", "httest",
+    "htten", "httet",
+    "werden", "werde", "wirst", "wird", "werdet", "geworden", "wrde",
+    "wrdest", "wrden", "wrdet",
+    "knnen", "kann", "kannst", "knnt", "konnte", "konntest", "konnten",
+    "konntet", "gekonnt", "knnte", "knntest", "knnten", "knntet",
+    "mssen", "muss", "musst", "msst", "musste", "musstest", "mussten",
+    "gemusst", "msste", "msstest", "mssten", "msstet",
+    "sollen", "soll", "sollst", "sollt", "sollte", "solltest", "solltet",
+    "sollten", "gesollt",
+    (const char*)0
+  };
+  /*}}}*/
+  static const char *en[]= /* auxiliary verbs */ /*{{{*/
+  {
+    "will", "shall", "cannot", "may", "need to", "would", "should",
+    "could", "might", "must", "ought", "ought to", "can't", "can",
+    (const char*)0
+  };
+  /*}}}*/
+  const char **list;
+
+  if (strncmp(docLanguage,"de",2)==0) list=de;
+  else list=en;
+  while (*list)
+  {
+    if (wordcmp(*list,word)==0) 
+    {
+      phraseEnd = word+strlen(*list);
+      return 1; 
+    }
+    else ++list;
+  }
+  return 0;
+}
+/*}}}*/
+
+static int tobeVerb(const char *word, size_t l) /*{{{*/
+{
+  static const char *de[]= /* Hilfsverb sein */ /*{{{*/
+  {
+    "sein", "bin", "bist", "ist", "sind", "seid", "war", "warst", "wart",
+    "waren", "gewesen", "wre", "wrst", "wr", "wren", "wrt", "wret",
+    (const char*)0
+  };
+  /*}}}*/
+  static const char *en[]= /* auxiliary verb to be */ /*{{{*/
+  {
+     "be", "being", "was", "were", "been", "are", "is", (const char*)0
+  };
+  /*}}}*/
+  const char **list;
+
+  if (strncmp(docLanguage,"de",2)==0) list=de;
+  else list=en;
+  while (*list) if (wordcmp(*list,word)==0) return 1; else ++list;
   return 0;
 }
 /*}}}*/
@@ -432,6 +639,40 @@ static int syll(const char *s, size_t l) /*{{{*/
 /*}}}*/
 /*}}}*/
 
+/* global style() variables */ /*{{{*/
+static int characters;
+static int syllables;
+static int words;
+static int shortwords;
+static int longwords;
+static int bigwords;
+static int sentences;
+static int questions;
+static int passiveSent;
+static int beginArticles;
+static int beginPronouns;
+static int pronouns;
+static int beginInterrogativePronouns;
+static int interrogativePronouns;
+static int beginConjunctions;
+static int conjunctions;
+static int nominalizations;
+static int prepositions;
+static int beginPrepositions;
+static int beginSubConjunctions;
+static int subConjunctions;
+static int auxVerbs;
+static int tobeVerbs;
+static int shortestLine,shortestLength;
+static int longestLine,longestLength;
+static int paragraphs;
+static int printLongSentences=0;
+static int printNomSentences=0;
+static int printPassiveSentences=0;
+static float printARI=0.0;
+static struct Hit lengths;
+/*}}}*/
+
 /**
  * Process one sentence.
  * @param str sentence
@@ -446,11 +687,14 @@ static void style(const char *str, size_t length, const char *file, int line) /*
   int sentWords=0;
   int sentLetters=0;
   int count;
+  int passive=0;
+  int nom=0;
   const char *s=str;
 
   if (length==0) { ++paragraphs; return; }
   assert(str!=(const char*)0);
   assert(length>=2);
+  phraseEnd = (const char*)0;
   while (*s)
   {
     if (inword)
@@ -463,13 +707,48 @@ static void style(const char *str, size_t length, const char *file, int line) /*
         if (count>=3) ++bigwords;
         else if (count==1) ++shortwords;
         if (wordLength>6) ++longwords;
-        if (firstWord)
+        if (s-wordLength > phraseEnd)
         {
-          if (article(s-wordLength,wordLength)) ++articles;
-          if (pronoun(s-wordLength,wordLength)) ++pronouns;
-          if (interrogativePronoun(s-wordLength,wordLength)) ++interrogativePronouns;
-          firstWord=0;
+          /* part of speech tagging-- order matters! */
+          if (article(s-wordLength,wordLength) && firstWord) ++beginArticles;
+          else if (pronoun(s-wordLength,wordLength))
+          {
+            ++pronouns;
+            if (firstWord) ++beginPronouns;
+          }
+          else if (interrogativePronoun(s-wordLength,wordLength))
+          { 
+            ++interrogativePronouns;
+            if (firstWord) ++beginInterrogativePronouns;
+          }
+          else if (conjunction(s-wordLength,wordLength)) 
+          { 
+            ++conjunctions;
+            if (firstWord) ++beginConjunctions;
+          }
+          else if (subConjunction(s-wordLength,wordLength)) 
+          { 
+            ++subConjunctions;
+            if (firstWord) ++beginSubConjunctions;
+          }
+          else if (preposition(s-wordLength,wordLength)) 
+          { 
+            ++prepositions;
+            if (firstWord) ++beginPrepositions;
+          }
+          else if (tobeVerb(s-wordLength,wordLength))
+          { 
+            ++passive;
+            ++tobeVerbs;
+          }
+          else if (auxVerb(s-wordLength,wordLength)) ++auxVerbs;
+          else if (nominalization(s-wordLength,wordLength))
+          { 
+            ++nom;
+            ++nominalizations;
+          }
         }
+        if (firstWord) firstWord = 0;
       }
       else
       {
@@ -528,7 +807,11 @@ static void style(const char *str, size_t length, const char *file, int line) /*
   }
   if (str[length-1]=='?') ++questions;
   noteHit(&lengths,sentWords);
-  if ((printLongSentences && sentWords>=printLongSentences) || (printARI && ari(sentLetters,sentWords,1)>printARI)) printf("%s:%d: %s\n",file,line,str);
+  if (passive) ++passiveSent;
+  if ((printLongSentences && sentWords>=printLongSentences) 
+      || (printARI && ari(sentLetters,sentWords,1)>printARI)
+      || (printPassiveSentences && passive)
+      || (printNomSentences && nom)) printf("%s:%d: %s\n",file,line,str);
 }
 /*}}}*/
 
@@ -543,6 +826,9 @@ int main(int argc, char *argv[]) /*{{{*/
     { "language", required_argument, 0, 'L' },
     { "print-ari", required_argument, 0, 'r' },
     { "version", no_argument, 0, 'v' },
+    { "print-passive", no_argument, 0, 'p' },
+    { "print-nom", no_argument, 0, 'N' },
+    { "print-nom-passive", no_argument, 0, 'N' },
     { (const char*)0, 0, 0, '\0' }
   };
   /*}}}*/
@@ -559,9 +845,10 @@ int main(int argc, char *argv[]) /*{{{*/
   if ((lc_ctype=getenv("LC_CTYPE"))==(const char*)0) lc_ctype="C";
   if ((docLanguage=getenv("LC_MESSAGES"))==(const char*)0) docLanguage="C";
 #endif
+  if (strcmp(docLanguage,"C")==0) docLanguage="en";
   if (strstr(lc_ctype,"8859-1")) lc_ctype_int=ISO_8859_1;
   else lc_ctype_int=ASCII;
-  while ((c=getopt_long(argc,argv,"l:L:r:h",lopts,(int*)0))!=EOF) switch(c)
+  while ((c=getopt_long(argc,argv,"l:L:r:hpnN",lopts,(int*)0))!=EOF) switch(c)
   {
     case 'l':
     {
@@ -582,6 +869,22 @@ int main(int argc, char *argv[]) /*{{{*/
       if (end==optarg || *end!='\0') usage=1;
       break;
     }
+    case 'p':
+    { 
+      printPassiveSentences=1; 
+      break;
+    }
+    case 'N':
+    { 
+      printNomSentences=1; 
+      break;
+    }
+    case 'n':
+    { 
+      printNomSentences=1; 
+      printPassiveSentences=1; 
+      break;
+    }
     case 'v': fputs("GNU style " VERSION "\n",stdout); exit(0);
     case 'h': usage=2; break;
     default: usage=1; break;
@@ -606,6 +909,13 @@ int main(int argc, char *argv[]) /*{{{*/
     fputs(HELP2,stdout);
     fputs(HELP3,stdout);
     fputs(HELP4,stdout);
+    fputs(HELP5,stdout);
+    fputs(HELP6,stdout);
+    fputs(HELP7,stdout);
+    fputs(HELP8,stdout);
+    fputs(HELP9,stdout);
+    fputs("\n",stdout);
+    fputs(BUGS,stdout);
     exit(0);
   }
   /*}}}*/
@@ -615,6 +925,7 @@ int main(int argc, char *argv[]) /*{{{*/
   {
     FILE *fp;
     if ((fp=fopen(argv[optind],"r"))==(FILE*)0) fprintf(stderr,OPENFAILED,argv[optind],strerror(errno));
+    else
     {
       sentence("style",catd,fp,argv[optind],style,docLanguage);
       fclose(fp);
@@ -637,11 +948,13 @@ int main(int argc, char *argv[]) /*{{{*/
     printf("        %s: %.1f\n",CL,coleman_liau(characters,words,sentences));
     printf("        %s: %.1f\n",FLESCH,flesch(syllables,words,sentences));
     printf("        %s: %.1f\n",FOG,fog(words,bigwords,sentences));
+#ifdef MIGHT_BE_WRONG
     printf("        %s: %.1f\n",WSFT,wstf(words,shortwords,longwords,bigwords,sentences));
     printf("        %s: %.1f = ",WSI,wheeler_smith(&wsg,words,bigwords,sentences));
     if (wsg==0) printf(BELOW5);
     else if (wsg==99) printf(AHEAD10);
     else printf(YEAR,wsg);
+#endif
     printf("        %s: %.1f = ",LIX,lix(&lixg,words,longwords,sentences));
     if (lixg==0) printf(BELOW5);
     else if (lixg==99) printf(AHEAD11);
@@ -660,7 +973,8 @@ int main(int argc, char *argv[]) /*{{{*/
     for (i=longLength,longSent=0; i<=lengths.size; ++i) longSent+=lengths.data[i];
     printf("        %d%% (%d) long sentences (at least %d words)\n",100*longSent/sentences,longSent,longLength);
     printf("        %d paragraphs, average length %.1f sentences\n",paragraphs,((double)sentences)/paragraphs);
-    printf("        %d questions\n",questions);
+    printf("        %d%% (%d) questions\n",100*questions/sentences,questions);
+    printf("        %d%% (%d) passive sentences\n",100*passiveSent/sentences,passiveSent);
     printf("        longest sent %d wds at sent %d; shortest sent %d wds at sent %d\n",longestLength,longestLine,shortestLength,shortestLine);
 
 /*
@@ -678,9 +992,27 @@ word usage:
         noun 25.0% (1) adj 25.0% (1) pron 25.0% (1)
         nominalizations   0 % (0)
 */
+        if (strncmp(docLanguage,"en",2)==0)
+        {
+          printf("word usage:\n");
+          printf("        verb types:\n");
+          printf("        to be (%d) auxiliary (%d) \n", tobeVerbs, auxVerbs);
+          printf("        types as %% of total:\n");
+          printf("        conjunctions %1.f% (%d) pronouns %1.f% (%d) prepositions %1.f% (%d)\n", 
+                  (100.0*(conjunctions+subConjunctions))/words, 
+                  conjunctions+subConjunctions, 
+                  (100.0*pronouns)/words, pronouns, (100.0*prepositions)/words,
+                  prepositions);
+          printf("        nominalizations %1.f% (%d)\n",
+                  (100.0*nominalizations)/words, nominalizations);
+        }
 
         printf("sentence beginnings:\n");
-        printf("        pronoun (%d) interrogative pronoun (%d) article (%d)\n",pronouns,interrogativePronouns,articles);
+        printf("        pronoun (%d) interrogative pronoun (%d) article (%d)\n",beginPronouns,beginInterrogativePronouns,beginArticles);
+        if (strncmp(docLanguage,"en",2)==0)
+        {
+          printf("        subordinating conjunction (%d) conjunction (%d) preposition (%d)\n", beginSubConjunctions,beginConjunctions,beginPrepositions);
+        }
 
 /*
         subject opener: noun (0) pron (1) pos (0) adj (0) art (0) tot 100%
